/* -*- mode: c; c-file-style: "gnu" -*-
 * mod_apm.c -- apm colorizers for CCZE
 * Copyright (C) 2002, 2003 Gergely Nagy <algernon@bonehunter.rulez.org>
 *
 * This file is part of ccze.
 *
 * ccze is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ccze is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <ccze.h>
#include <string.h>
#include <stdlib.h>

static void ccze_apm_setup (void);
static void ccze_apm_shutdown (void);
static int ccze_apm_handle (const char *str, size_t length, char **rest);

static pcre2_code *reg_apm;

static char *
ccze_apm_process (pcre2_match_data *offsets)
{
  char *battery, *charge, *rate, *stuff1, *elapsed, *remain;
  char *stuff2, *ret;
  size_t l;
  
  pcre2_substring_get_bynumber (offsets, 1, (unsigned char **)&battery, &l);
  pcre2_substring_get_bynumber (offsets, 2, (unsigned char **)&charge, &l);
  pcre2_substring_get_bynumber (offsets, 4, (unsigned char **)&rate, &l);
  pcre2_substring_get_bynumber (offsets, 5, (unsigned char **)&stuff1, &l);
  pcre2_substring_get_bynumber (offsets, 6, (unsigned char **)&elapsed, &l);
  pcre2_substring_get_bynumber (offsets, 7, (unsigned char **)&remain, &l);
  pcre2_substring_get_bynumber (offsets, 8, (unsigned char **)&stuff2, &l);
        
  ccze_addstr (CCZE_COLOR_DEFAULT, "Battery:");
  ccze_space ();
  ccze_addstr (CCZE_COLOR_PERCENTAGE, battery);
  ccze_addstr (CCZE_COLOR_DEFAULT, "%,");
  ccze_space ();
  ccze_addstr (CCZE_COLOR_SYSTEMWORD, charge);
  ccze_space ();
  ccze_addstr (CCZE_COLOR_DEFAULT, "(");
  ccze_addstr (CCZE_COLOR_PERCENTAGE, rate);
  ccze_addstr (CCZE_COLOR_DEFAULT, "%");
  ccze_space ();
  ccze_addstr (CCZE_COLOR_DEFAULT, stuff1);
  ccze_space ();
  ccze_addstr (CCZE_COLOR_DATE, elapsed);
  ccze_addstr (CCZE_COLOR_DEFAULT, "),");
  ccze_space ();
  ccze_addstr (CCZE_COLOR_DATE, remain);
  ccze_space ();

  ret = strdup (stuff2);
  pcre2_substring_free (battery);
  pcre2_substring_free (charge);
  pcre2_substring_free (rate);
  pcre2_substring_free (stuff1);
  pcre2_substring_free (elapsed);
  pcre2_substring_free (remain);
  pcre2_substring_free (stuff2);
  
  return ret;
}

static void
ccze_apm_setup (void)
{
  int error;
  size_t errptr;

  reg_apm = pcre2_compile
    ("Battery: (-?\\d*)%, ((.*)charging) \\((-?\\d*)% ([^ ]*) "
     "(\\d*:\\d*:\\d*)\\), (\\d*:\\d*:\\d*) (.*)", PCRE2_ZERO_TERMINATED,
     0, &error, &errptr, NULL);
}

static void
ccze_apm_shutdown (void)
{
  pcre2_code_free (reg_apm);
}

static int
ccze_apm_handle (const char *str, size_t length, char **rest)
{
  pcre2_match_data *offsets;
  
  offsets = pcre2_match_data_create (99, NULL);
  if (pcre2_match (reg_apm, str, length, 0, 0, offsets, NULL) >= 0)
    {
      if (rest)
	*rest = ccze_apm_process (offsets);
      else
	ccze_apm_process (offsets);
      
      pcre2_match_data_free (offsets);
      return 1;
    }

  pcre2_match_data_free (offsets);
    
  return 0;
}

CCZE_DEFINE_PLUGIN (apm, PARTIAL, "Coloriser for APM sub-logs.");
