/* -*- mode: c; c-file-style: "gnu" -*-
 * mod_distcc.c -- distcc.log colorizers for CCZE
 * Copyright (C) 2003 Gergely Nagy <algernon@bonehunter.rulez.org>
 *
 * This file is part of ccze.
 *
 * ccze is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ccze is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <ccze.h>
#include <string.h>
#include <stdlib.h>

#include "ccze-compat.h"

static void ccze_distcc_setup (void);
static void ccze_distcc_shutdown (void);
static int ccze_distcc_handle (const char *str, size_t length, char **rest);

static pcre2_code *reg_distcc;

static char *
ccze_distcc_process (pcre2_match_data *offsets)
{
  char *pid, *func, *rest, *ret;
  size_t l;
    
  pcre2_substring_get_bynumber (offsets, 1, (unsigned char **)&pid, &l);
  pcre2_substring_get_bynumber (offsets, 2, (unsigned char **)&func, &l);
  pcre2_substring_get_bynumber (offsets, 3, (unsigned char **)&rest, &l);
  
  ccze_addstr (CCZE_COLOR_PROC, "distccd");
  ccze_addstr (CCZE_COLOR_PIDB, "[");
  ccze_addstr (CCZE_COLOR_PID, pid);
  ccze_addstr (CCZE_COLOR_PIDB, "]");
  ccze_space ();

  if (func && func[0] != '\0')
    {
      ccze_addstr (CCZE_COLOR_KEYWORD, func);
      ccze_space ();
    }

  ret = strdup (rest);
  pcre2_substring_free (pid);
  pcre2_substring_free (func);
  pcre2_substring_free (rest);
  
  return ret;
}

static void
ccze_distcc_setup (void)
{
  int error;
  size_t errptr;

  reg_distcc = pcre2_compile ("^distccd\\[(\\d+)\\] (\\([^\\)]+\\))? ?(.*)",
                              PCRE2_ZERO_TERMINATED, 0, &error, &errptr, NULL);
}

static void
ccze_distcc_shutdown (void)
{
  pcre2_code_free (reg_distcc);
}

static int
ccze_distcc_handle (const char *str, size_t length, char **rest)
{
  pcre2_match_data *offsets;
  
  offsets = pcre2_match_data_create (99, NULL);
  if (pcre2_match (reg_distcc, str, length, 0, 0, offsets, NULL) >= 0)
    {
      *rest = ccze_distcc_process (offsets);
      pcre2_match_data_free (offsets);
      return 1;
    }

  pcre2_match_data_free (offsets);
  
  return 0;
}

CCZE_DEFINE_PLUGIN (distcc, FULL, "Coloriser for distcc(1) logs.");
