/* -*- mode: c; c-file-style: "gnu" -*-
 * mod_exim.c -- Exim log-coloriser module for CCZE
 * Copyright (C) 2002, 2003 Gergely Nagy <algernon@bonehunter.rulez.org>
 *
 * This file is part of ccze.
 *
 * ccze is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ccze is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <ccze.h>
#include <stdlib.h>
#include <string.h>

static void ccze_exim_setup (void);
static void ccze_exim_shutdown (void);
static int ccze_exim_handle (const char *str, size_t length, char **rest);

static pcre2_code *reg_exim, *reg_exim_actiontype, *reg_exim_uniqn;

static char *
ccze_exim_process (pcre2_match_data *offsets)
{
  char *date, *msg=NULL, *action=NULL, *uniqn=NULL, *ret=NULL, *msgfull;
  pcre2_match_data *offsets2;
  size_t l;
  ccze_color_t color = CCZE_COLOR_UNKNOWN;
  
  pcre2_substring_get_bynumber (offsets, 1, (unsigned char **)&date, &l);
  pcre2_substring_get_bynumber (offsets, 2, (unsigned char **)&msgfull, &l);

  offsets2 = pcre2_match_data_create (99, NULL);
  if (pcre2_match (reg_exim_actiontype, msgfull, strlen (msgfull),
                   0, 0, offsets2, NULL) >= 0)
    {
      pcre2_substring_get_bynumber (offsets2, 1,
			            (unsigned char **)&uniqn, &l);
      pcre2_substring_get_bynumber (offsets2, 2,
			            (unsigned char **)&action, &l);
      pcre2_substring_get_bynumber (offsets2, 3,
			            (unsigned char **)&msg, &l);
      if (action[0] == '<')
	color = CCZE_COLOR_INCOMING;
      else if (action[1] == '>')
	color = CCZE_COLOR_OUTGOING;
      else if (action[0] == '=' || action[0] == '*')
	color = CCZE_COLOR_ERROR;
    }
  else if (pcre2_match (reg_exim_uniqn, msgfull, strlen (msgfull),
                        0, 0, offsets2, NULL) >= 0)
    {
      pcre2_substring_get_bynumber (offsets2, 1,
			            (unsigned char **)&uniqn, &l);
      pcre2_substring_get_bynumber (offsets2, 2,
			            (unsigned char **)&msg, &l);
    }
  else
    ret = strdup (msgfull);
  
  ccze_print_date (date);
  ccze_space ();

  if (uniqn && uniqn[0])
    {
      ccze_addstr (CCZE_COLOR_UNIQN, uniqn);
      ccze_space();
    }
  
  if (action && action[0])
    {
      ccze_addstr (color, action);
      ccze_space();
    }

  if (!ret)
    ret = strdup (msg);

  pcre2_substring_free (date);
  pcre2_substring_free (msgfull);
  pcre2_substring_free (uniqn);
  pcre2_substring_free (action);
  pcre2_substring_free (msg);
  pcre2_match_data_free (offsets2);

  return ret;
}

static void
ccze_exim_setup (void)
{
  int error;
  size_t errptr;

  reg_exim = pcre2_compile
    ("^(\\d{4}-\\d{2}-\\d{2}\\s\\d{2}:\\d{2}:\\d{2})\\s(.*)$",
     PCRE2_ZERO_TERMINATED, 0, &error, &errptr, NULL);

  reg_exim_actiontype = pcre2_compile
    ("^(\\S{16})\\s([<=\\*][=>\\*])\\s(\\S+.*)$", PCRE2_ZERO_TERMINATED,
     0, &error, &errptr, NULL);
  reg_exim_uniqn = pcre2_compile ("^(\\S{16})\\s(.*)$", PCRE2_ZERO_TERMINATED,
                                  0, &error, &errptr, NULL);
}

static void
ccze_exim_shutdown (void)
{
  pcre2_code_free (reg_exim);
  pcre2_code_free (reg_exim_actiontype);
  pcre2_code_free (reg_exim_uniqn);
}

static int
ccze_exim_handle (const char *str, size_t length, char **rest)
{
  pcre2_match_data *offsets;
  
  offsets = pcre2_match_data_create (99, NULL);
  if (pcre2_match (reg_exim, str, length, 0, 0, offsets, NULL) >= 0)
    {
      *rest = ccze_exim_process (offsets);
      pcre2_match_data_free (offsets);
      return 1;
    }

  pcre2_match_data_free (offsets);
  
  return 0;
}

CCZE_DEFINE_PLUGIN (exim, FULL, "Coloriser for exim logs.");
