/* -*- mode: c; c-file-style: "gnu" -*-
 * mod_fetchmail.c -- fetchmail-related colorizers for CCZE
 * Copyright (C) 2002, 2003 Gergely Nagy <algernon@bonehunter.rulez.org>
 *
 * This file is part of ccze.
 *
 * ccze is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ccze is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <ccze.h>
#include <string.h>
#include <stdlib.h>

static void ccze_fetchmail_setup (void);
static void ccze_fetchmail_shutdown (void);
static int ccze_fetchmail_handle (const char *str, size_t length, char **rest);

static pcre2_code *reg_fetchmail;

static char *
ccze_fetchmail_process (pcre2_match_data *offsets)
{
  char *start, *addy, *current, *full, *rest, *ret;
  size_t l;
  
  pcre2_substring_get_bynumber (offsets, 1, (unsigned char **)&start, &l);
  pcre2_substring_get_bynumber (offsets, 2, (unsigned char **)&addy, &l);
  pcre2_substring_get_bynumber (offsets, 3, (unsigned char **)&current, &l);
  pcre2_substring_get_bynumber (offsets, 4, (unsigned char **)&full, &l);
  pcre2_substring_get_bynumber (offsets, 5, (unsigned char **)&rest, &l);

  ccze_addstr (CCZE_COLOR_DEFAULT, start);
  ccze_space ();
  ccze_addstr (CCZE_COLOR_EMAIL, addy);
  ccze_addstr (CCZE_COLOR_DEFAULT, ":");
  ccze_addstr (CCZE_COLOR_NUMBERS, current);
  ccze_space ();
  ccze_addstr (CCZE_COLOR_DEFAULT, "of");
  ccze_space ();
  ccze_addstr (CCZE_COLOR_NUMBERS, full);
  ccze_space ();
  
  ret = strdup (rest);
  pcre2_substring_free (start);
  pcre2_substring_free (addy);
  pcre2_substring_free (current);
  pcre2_substring_free (full);
  pcre2_substring_free (rest);
  
  return ret;
}

static void
ccze_fetchmail_setup (void)
{
  int error;
  size_t errptr;

  reg_fetchmail = pcre2_compile
    ("(reading message) ([^@]*@[^:]*):([0-9]*) of ([0-9]*) (.*)",
     PCRE2_ZERO_TERMINATED, 0, &error, &errptr, NULL);
}

static void
ccze_fetchmail_shutdown (void)
{
  pcre2_code_free (reg_fetchmail);
}

static int
ccze_fetchmail_handle (const char *str, size_t length, char **rest)
{
  pcre2_match_data *offsets;
  
  offsets = pcre2_match_data_create (99, NULL);
  if (pcre2_match (reg_fetchmail, str, length, 0, 0, offsets, NULL) >= 0)
    {
      if (rest)
	*rest = ccze_fetchmail_process (offsets);
      else
	ccze_fetchmail_process (offsets);
      
      pcre2_match_data_free (offsets);
      return 1;
    }

  pcre2_match_data_free (offsets);
  
  return 0;
}

CCZE_DEFINE_PLUGIN (fetchmail, PARTIAL,
		    "Coloriser for fetchmail(1) sub-logs.");
