/* -*- mode: c; c-file-style: "gnu" -*-
 * mod_ftpstats.c -- Stats coloriser for CCZE.
 * Copyright (C) 2003 Gergely Nagy <algernon@bonehunter.rulez.org>
 *
 * This file is part of ccze.
 *
 * ccze is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ccze is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <ccze.h>
#include <string.h>
#include <stdlib.h>

#include "ccze-compat.h"

static void ccze_ftpstats_setup (void);
static void ccze_ftpstats_shutdown (void);
static int ccze_ftpstats_handle (const char *str, size_t length, char **rest);

static pcre2_code *reg_ftpstats;

static char *
ccze_ftpstats_process (pcre2_match_data *offsets)
{
  char *date, *sessionid, *user, *host, *type, *size, *duration, *file;
  size_t l;

  pcre2_substring_get_bynumber (offsets, 1, (unsigned char **)&date, &l);
  pcre2_substring_get_bynumber (offsets, 2, (unsigned char **)&sessionid, &l);
  pcre2_substring_get_bynumber (offsets, 3, (unsigned char **)&user, &l);
  pcre2_substring_get_bynumber (offsets, 4, (unsigned char **)&host, &l);
  pcre2_substring_get_bynumber (offsets, 5, (unsigned char **)&type, &l);
  pcre2_substring_get_bynumber (offsets, 6, (unsigned char **)&size, &l);
  pcre2_substring_get_bynumber (offsets, 7, (unsigned char **)&duration, &l);
  pcre2_substring_get_bynumber (offsets, 8, (unsigned char **)&file, &l);

  ccze_print_date (date);
  ccze_space ();
  ccze_addstr (CCZE_COLOR_UNIQN, sessionid);
  ccze_space ();
  ccze_addstr (CCZE_COLOR_USER, user);
  ccze_space ();
  ccze_addstr (CCZE_COLOR_HOST, host);
  ccze_space ();
  ccze_addstr (CCZE_COLOR_FTPCODES, type);
  ccze_space ();
  ccze_addstr (CCZE_COLOR_GETSIZE, size);
  ccze_space ();
  ccze_addstr (CCZE_COLOR_DATE, duration);
  ccze_space ();
  ccze_addstr (CCZE_COLOR_DIR, file);
  ccze_newline ();

  pcre2_substring_free (file);
  pcre2_substring_free (duration);
  pcre2_substring_free (size);
  pcre2_substring_free (type);
  pcre2_substring_free (host);
  pcre2_substring_free (user);
  pcre2_substring_free (sessionid);
  pcre2_substring_free (date);
  
  return NULL;
}

static void
ccze_ftpstats_setup (void)
{
  int error;
  size_t errptr;

  reg_ftpstats = pcre2_compile
    ("^(\\d{9,10})\\s([\\da-f]+\\.[\\da-f]+)\\s([^\\s]+)\\s([^\\s]+)"
     "\\s(U|D)\\s(\\d+)\\s(\\d+)\\s(.*)$",
     PCRE2_ZERO_TERMINATED, 0, &error, &errptr, NULL);
}

static void
ccze_ftpstats_shutdown (void)
{
  pcre2_code_free (reg_ftpstats);
}

static int
ccze_ftpstats_handle (const char *str, size_t length, char **rest)
{
  pcre2_match_data *offsets;
  
  offsets = pcre2_match_data_create (99, NULL);
  if (pcre2_match (reg_ftpstats, str, length, 0, 0, offsets, NULL) >= 0)
    {
      *rest = ccze_ftpstats_process (offsets);
      pcre2_match_data_free (offsets);
      return 1;
    }

  pcre2_match_data_free (offsets);
  
  return 0;
}

CCZE_DEFINE_PLUGIN (ftpstats, FULL, "Coloriser for ftpstats (pure-ftpd) logs.");
