/* -*- mode: c; c-file-style: "gnu" -*-
 * mod_icecast.c -- icecast/{usage,icecast}.log colorizers for CCZE
 * Copyright (C) 2003 Gergely Nagy <algernon@bonehunter.rulez.org>
 *
 * This file is part of ccze.
 *
 * ccze is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ccze is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <ccze.h>
#include <string.h>
#include <stdlib.h>

#include "ccze-compat.h"

static void ccze_icecast_setup (void);
static void ccze_icecast_shutdown (void);
static int ccze_icecast_handle (const char *str, size_t length, char **rest);

static pcre2_code *reg_icecast, *reg_icecast_usage;

static char *
ccze_icecast_process (pcre2_match_data *offsets)
{
  char *date = NULL, *admin = NULL, *threadno = NULL, *thread = NULL;
  char *rest = NULL, *ret;
  size_t l;
  
  pcre2_substring_get_bynumber (offsets, 1, (unsigned char **)&date, &l);
  pcre2_substring_get_bynumber (offsets, 2, (unsigned char **)&admin, &l);
  pcre2_substring_get_bynumber (offsets, 4, (unsigned char **)&threadno, &l);
  pcre2_substring_get_bynumber (offsets, 5, (unsigned char **)&thread, &l);
  pcre2_substring_get_bynumber (offsets, 6, (unsigned char **)&rest, &l);
  
  ccze_addstr (CCZE_COLOR_DATE, date);
  ccze_space ();

  if (admin && admin[0] != '\0')
    {
      ccze_addstr (CCZE_COLOR_KEYWORD, admin);
      ccze_space ();
      ccze_addstr (CCZE_COLOR_PIDB, "[");
      ccze_addstr (CCZE_COLOR_HOST, thread);
      ccze_addstr (CCZE_COLOR_PIDB, "]");
    }
  else
    {
      ccze_addstr (CCZE_COLOR_PIDB, "[");
      ccze_addstr (CCZE_COLOR_NUMBERS, threadno);
      ccze_addstr (CCZE_COLOR_DEFAULT, ":");
      ccze_addstr (CCZE_COLOR_KEYWORD, thread);
      ccze_addstr (CCZE_COLOR_PIDB, "]");
    }
  ccze_space ();

  ret = strdup (rest);
  pcre2_substring_free (date);
  pcre2_substring_free (admin);
  pcre2_substring_free (threadno);
  pcre2_substring_free (thread);
  pcre2_substring_free (rest);
  
  return ret;
}

static char *
ccze_icecast_usage_process (pcre2_match_data *offsets)
{
  char *date, *threadno, *thread, *date2, *bw, *src;
  char *unit, *clients, *admins;
  size_t l;
    
  pcre2_substring_get_bynumber (offsets, 1, (unsigned char **)&date, &l);
  pcre2_substring_get_bynumber (offsets, 3, (unsigned char **)&threadno, &l);
  pcre2_substring_get_bynumber (offsets, 4, (unsigned char **)&thread, &l);
  pcre2_substring_get_bynumber (offsets, 5, (unsigned char **)&date2, &l);
  pcre2_substring_get_bynumber (offsets, 6, (unsigned char **)&bw, &l);
  pcre2_substring_get_bynumber (offsets, 7, (unsigned char **)&unit, &l);
  pcre2_substring_get_bynumber (offsets, 8, (unsigned char **)&src, &l);
  pcre2_substring_get_bynumber (offsets, 9, (unsigned char **)&clients, &l);
  pcre2_substring_get_bynumber (offsets, 10, (unsigned char **)&admins, &l);
  
  
  ccze_addstr (CCZE_COLOR_DATE, date);
  ccze_space ();

  ccze_addstr (CCZE_COLOR_PIDB, "[");
  ccze_addstr (CCZE_COLOR_NUMBERS, threadno);
  ccze_addstr (CCZE_COLOR_DEFAULT, ":");
  ccze_addstr (CCZE_COLOR_KEYWORD, thread);
  ccze_addstr (CCZE_COLOR_PIDB, "]");
  ccze_space ();

  ccze_addstr (CCZE_COLOR_DATE, date2);
  ccze_space ();

  ccze_addstr (CCZE_COLOR_KEYWORD, "Bandwidth:");
  ccze_addstr (CCZE_COLOR_NUMBERS, bw);
  ccze_addstr (CCZE_COLOR_DEFAULT, unit);
  ccze_space ();

  ccze_addstr (CCZE_COLOR_KEYWORD, "Sources:");
  ccze_addstr (CCZE_COLOR_NUMBERS, src);
  ccze_space ();

  ccze_addstr (CCZE_COLOR_KEYWORD, "Clients:");
  ccze_addstr (CCZE_COLOR_NUMBERS, clients);
  ccze_space ();

  ccze_addstr (CCZE_COLOR_KEYWORD, "Admins:");
  ccze_addstr (CCZE_COLOR_NUMBERS, admins);

  ccze_newline ();
  
  pcre2_substring_free (date);
  pcre2_substring_free (threadno);
  pcre2_substring_free (thread);
  pcre2_substring_free (date2);
  pcre2_substring_free (bw);
  pcre2_substring_free (unit);
  pcre2_substring_free (src);
  pcre2_substring_free (clients);
  pcre2_substring_free (admins);
  
  return NULL;
}

static void
ccze_icecast_setup (void)
{
  int error;
  size_t errptr;

  reg_icecast = pcre2_compile ("^(\\[\\d+/.../\\d+:\\d+:\\d+:\\d+\\]) "
                               "(Admin)? *(\\[(\\d+)?:?([^\\]]*)\\]) (.*)$",
                               PCRE2_ZERO_TERMINATED, 0, &error, &errptr, NULL);

  reg_icecast_usage = pcre2_compile ("^(\\[\\d+/.../\\d+:\\d+:\\d+:\\d+\\]) "
                                     "(\\[(\\d+):([^\\]]*)\\]) "
                                     "(\\[\\d+/.../\\d+:\\d+:\\d+:\\d+\\]) "
                                     "Bandwidth:([\\d\\.]+)([^ ]*) "
                                     "Sources:(\\d+) "
                                     "Clients:(\\d+) Admins:(\\d+)",
                                     PCRE2_ZERO_TERMINATED, 0,
                                     &error, &errptr, NULL);
}

static void
ccze_icecast_shutdown (void)
{
  pcre2_code_free (reg_icecast);
  pcre2_code_free (reg_icecast_usage);
}

static int
ccze_icecast_handle (const char *str, size_t length, char **rest)
{
  pcre2_match_data *offsets;

  offsets = pcre2_match_data_create (99, NULL);
  if (pcre2_match (reg_icecast_usage, str, length, 0, 0, offsets, NULL) >= 0)
    {
      *rest = ccze_icecast_usage_process (offsets);
      pcre2_match_data_free (offsets);
      return 1;
    }
  
  if (pcre2_match (reg_icecast, str, length, 0, 0, offsets, NULL) >= 0)
    {
      *rest = ccze_icecast_process (offsets);
      pcre2_match_data_free (offsets);
      return 1;
    }

  pcre2_match_data_free (offsets);
  
  return 0;
}

CCZE_DEFINE_PLUGIN (icecast, FULL, "Coloriser for Icecast(8) logs.");
