/* -*- mode: c; c-file-style: "gnu" -*-
 * mod_super.c -- super-related colorizers for CCZE
 * Copyright (C) 2002, 2003 Gergely Nagy <algernon@bonehunter.rulez.org>
 *
 * This file is part of ccze.
 *
 * ccze is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ccze is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <ccze.h>
#include <stdlib.h>

static void ccze_super_setup (void);
static void ccze_super_shutdown (void);
static int ccze_super_handle (const char *str, size_t length, char **rest);

static pcre2_code *reg_super;

static char *
ccze_super_process (pcre2_match_data *offsets)
{
  char *email, *date, *space, *suptag, *other;
  size_t l;

  pcre2_substring_get_bynumber (offsets, 1, (unsigned char **)&email, &l);
  pcre2_substring_get_bynumber (offsets, 2, (unsigned char **)&date, &l);
  pcre2_substring_get_bynumber (offsets, 3, (unsigned char **)&space, &l);
  pcre2_substring_get_bynumber (offsets, 4, (unsigned char **)&suptag, &l);
  pcre2_substring_get_bynumber (offsets, 5, (unsigned char **)&other, &l);

  ccze_addstr (CCZE_COLOR_EMAIL, email);
  ccze_space ();
  ccze_addstr (CCZE_COLOR_DATE, date);
  ccze_addstr (CCZE_COLOR_DEFAULT, space);
  ccze_addstr (CCZE_COLOR_PROC, suptag);
  ccze_space ();
  ccze_addstr (CCZE_COLOR_PIDB, "(");
  ccze_addstr (CCZE_COLOR_DEFAULT, other);
  ccze_addstr (CCZE_COLOR_PIDB, ")");

  ccze_newline ();

  pcre2_substring_free (email);
  pcre2_substring_free (date);
  pcre2_substring_free (space);
  pcre2_substring_free (suptag);
  pcre2_substring_free (other);
  
  return NULL;
}

static void
ccze_super_setup (void)
{
  int error;
  size_t errptr;

  reg_super = pcre2_compile
    ("^(\\S+)\\s(\\w+\\s+\\w+\\s+\\d+\\s+\\d+:\\d+:\\d+\\s+\\d+)"
     "(\\s+)(\\S+)\\s\\(([^\\)]+)\\)", PCRE2_ZERO_TERMINATED,
     0, &error, &errptr, NULL);
}

static void
ccze_super_shutdown (void)
{
  pcre2_code_free (reg_super);
}

static int
ccze_super_handle (const char *str, size_t length, char **rest)
{
  pcre2_match_data *offsets;
  
  offsets = pcre2_match_data_create (99, NULL);
  if (pcre2_match (reg_super, str, length, 0, 0, offsets, NULL) >= 0)
    {
      *rest = ccze_super_process (offsets);
      pcre2_match_data_free (offsets);
      return 1;
    }

  pcre2_match_data_free (offsets);
  
  return 0;
}

CCZE_DEFINE_PLUGIN (super, FULL, "Coloriser for super(1) logs.");
