/* -*- mode: c; c-file-style: "gnu" -*-
 * mod_vstpd.c -- VSFTPd-related colorizers for CCZE
 * Copyright (C) 2002, 2003 Gergely Nagy <algernon@bonehunter.rulez.org>
 *
 * This file is part of ccze.
 *
 * ccze is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ccze is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <ccze.h>
#include <string.h>
#include <stdlib.h>

static void ccze_vsftpd_setup (void);
static void ccze_vsftpd_shutdown (void);
static int ccze_vsftpd_handle (const char *str, size_t length, char **rest);

static pcre2_code *reg_vsftpd;

static char *
ccze_vsftpd_log_process (pcre2_match_data *offsets)
{
  char *date, *sspace, *pid, *user, *other, *ret;
  size_t l;

  pcre2_substring_get_bynumber (offsets, 1, (unsigned char **)&date, &l);
  pcre2_substring_get_bynumber (offsets, 2, (unsigned char **)&sspace, &l);
  pcre2_substring_get_bynumber (offsets, 3, (unsigned char **)&pid, &l);
  pcre2_substring_get_bynumber (offsets, 5, (unsigned char **)&user, &l);
  pcre2_substring_get_bynumber (offsets, 6, (unsigned char **)&other, &l);
  
  ccze_addstr (CCZE_COLOR_DATE, date);
  ccze_addstr (CCZE_COLOR_DEFAULT, sspace);

  ccze_addstr (CCZE_COLOR_PIDB, "[");
  ccze_addstr (CCZE_COLOR_DEFAULT, "pid ");
  ccze_addstr (CCZE_COLOR_PID, pid);
  ccze_addstr (CCZE_COLOR_PIDB, "]");
  ccze_space();

  if (*user)
    {
      ccze_addstr (CCZE_COLOR_PIDB, "[");
      ccze_addstr (CCZE_COLOR_USER, user);
      ccze_addstr (CCZE_COLOR_PIDB, "]");
      ccze_space ();
    }

  ret = strdup (other);
  pcre2_substring_free (date);
  pcre2_substring_free (sspace);
  pcre2_substring_free (pid);
  pcre2_substring_free (user);
  pcre2_substring_free (other);
  
  return ret;
}

static void
ccze_vsftpd_setup (void)
{
  int error;
  size_t errptr;

  reg_vsftpd = pcre2_compile
    ("^(\\S+\\s+\\S+\\s+\\d{1,2}\\s+\\d{1,2}:\\d{1,2}:\\d{1,2}\\s+\\d+)"
     "(\\s+)\\[pid (\\d+)\\]\\s+(\\[(\\S+)\\])?\\s*(.*)$",
     PCRE2_ZERO_TERMINATED, 0, &error, &errptr, NULL);
}

static void
ccze_vsftpd_shutdown (void)
{
  pcre2_code_free (reg_vsftpd);
}

static int
ccze_vsftpd_handle (const char *str, size_t length, char **rest)
{
  pcre2_match_data *offsets;
  
  offsets = pcre2_match_data_create (99, NULL);
  if (pcre2_match (reg_vsftpd, str, length, 0, 0, offsets, NULL) >= 0)
    {
      *rest = ccze_vsftpd_log_process (offsets);
      pcre2_match_data_free (offsets);
      return 1;
    }

  pcre2_match_data_free (offsets);
  
  return 0;
}

CCZE_DEFINE_PLUGIN (vsftpd, FULL, "Coloriser for vsftpd(8) logs.");
